@@ -162,12 +162,11 @@ pub fn crawl_l3_event(
     duration: Option<u64>,
 ) {
     let func = match exchange {
-        "binance" => panic!("Binance does NOT provide level3 orderbook data"),
         "bitfinex" => crawlers::bitfinex::crawl_l3_event,
         "bitstamp" => crawlers::bitstamp::crawl_l3_event,
         "coinbase_pro" => crawlers::coinbase_pro::crawl_l3_event,
         "kucoin" => crawlers::kucoin::crawl_l3_event,
-        _ => panic!("Unknown exchange {}", exchange),
+        _ => panic!("{} does NOT have level3 orderbook data", exchange),
     };
     let handle = func(market_type, symbols, on_msg, duration);
     if let Some(h) = handle {
@@ -318,3 +317,21 @@ pub fn crawl_funding_rate(
     };
     func(market_type, symbols, on_msg, duration);
 }
+
+/// Crawl candlestick(i.e., OHLCV) data.
+pub fn crawl_candlestick(
+    exchange: &str,
+    market_type: MarketType,
+    symbol_interval_list: Option<&[(String, usize)]>,
+    on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
+    duration: Option<u64>,
+) {
+    let func = match exchange {
+        "binance" => crawlers::binance::crawl_candlestick,
+        _ => panic!("Unknown exchange {}", exchange),
+    };
+    let handle = func(market_type, symbol_interval_list, on_msg, duration);
+    if let Some(h) = handle {
+        h.join().expect("The thread panicked");
+    }
+}
