@@ -37,6 +37,18 @@ fn test_crawl_l2_event(market_type: MarketType, symbol: &str) {
     )
 }
 
+#[test_case(MarketType::InverseSwap, "btcusd")]
+#[test_case(MarketType::LinearSwap, "cmt_btcusdt")]
+fn test_crawl_l2_topk(market_type: MarketType, symbol: &str) {
+    gen_test_code!(
+        crawl_l2_topk,
+        EXCHANGE_NAME,
+        market_type,
+        symbol,
+        MessageType::L2TopK
+    )
+}
+
 #[test_case(MarketType::Spot, "btc_usdt")]
 #[test_case(MarketType::InverseSwap, "btcusd")]
 #[test_case(MarketType::LinearSwap, "cmt_btcusdt")]
