@@ -49,13 +49,9 @@ macro_rules! gen_test_snapshot_code {
 #[allow(unused_macros)]
 macro_rules! gen_test_snapshot_without_symbol_code {
     ($crawl_func:ident, $exchange:expr, $market_type:expr, $msg_type:expr) => {{
-        thread_local! {
-            static MESSAGES: RefCell<Vec<Message>> = RefCell::new(Vec::new());
-        }
-
-        let on_msg = Arc::new(Mutex::new(|msg: Message| {
-            MESSAGES.with(|messages| messages.borrow_mut().push(msg))
-        }));
+        let messages: Arc<Mutex<Vec<Message>>> = Arc::new(Mutex::new(Vec::new()));
+        let messages_clone = messages.clone();
+        let on_msg = Arc::new(Mutex::new(move |msg: Message| messages_clone.lock().unwrap().push(msg)));
         let symbols = if $market_type == MarketType::Spot {
             let spot_symbols = fetch_symbols_retry($exchange, $market_type);
             get_hot_spot_symbols($exchange, &spot_symbols)
@@ -64,15 +60,27 @@ macro_rules! gen_test_snapshot_without_symbol_code {
         };
         $crawl_func($exchange, $market_type, Some(&symbols), on_msg, Some(0));
 
-        MESSAGES.with(|slf| {
-            let messages = slf.borrow();
+        let messages = messages.lock().unwrap();
+        assert!(!messages.is_empty());
+        assert_eq!(messages.len(), symbols.len());
 
-            assert!(!messages.is_empty());
-            assert_eq!(messages.len(), symbols.len());
+        assert_eq!(messages[0].exchange, $exchange.to_string());
+        assert_eq!(messages[0].market_type, $market_type);
+        assert_eq!(messages[0].msg_type, $msg_type);
+    }};
+}
 
-            assert_eq!(messages[0].exchange, $exchange.to_string());
-            assert_eq!(messages[0].market_type, $market_type);
-            assert_eq!(messages[0].msg_type, $msg_type);
-        });
+#[allow(unused_macros)]
+macro_rules! gen_test_crawl_candlestick {
+    ($exchange:expr, $market_type:expr) => {{
+        let messages: Arc<Mutex<Vec<Message>>> = Arc::new(Mutex::new(Vec::new()));
+        let messages_clone = messages.clone();
+        let on_msg = Arc::new(Mutex::new(move |msg: Message| messages_clone.lock().unwrap().push(msg)));
+        crawl_candlestick($exchange, $market_type, None, on_msg, Some(0));
+        let messages = messages.lock().unwrap();
+        assert!(!messages.is_empty());
+        assert_eq!(messages[0].exchange, EXCHANGE_NAME.to_string());
+        assert_eq!(messages[0].market_type, $market_type);
+        assert_eq!(messages[0].msg_type, MessageType::Candlestick);
     }};
 }
